#include <iostream>
#include <fstream>
#include <string>
#include <math.h>

//! Ist x in der konvexen Huelle der vier Kontrollpunkte p?
bool inside (  int p [4][2], int x [2] )
{
  int count = 0;
  // Count all segments crossing the line y [1] = x [1] left of x [0]
  for ( int i = 0; i < 4; ++i ) {
    if ( (p [i][1] < x [1] && p [(i+1)%4][1] >= x [1]) ||  (p [i][1] >= x [1] && p [(i+1)%4][1] < x [1]) ) {
      double lambda = static_cast<double> (x [1] - p [i][1]) / (p [(i+1)%4][1] - p [i][1]);
      if ( p [i][0] + lambda * (p [(i+1)%4][0] - p [i][0]) < x [0] ) {
	++count;
      }
    }
  }
  return ( count % 2 ) != 0;
}

//! Zeichnet vier Kontrollpunkte p und die konvexe Huelle in img
void plotbox ( int img [700][850][3], int p [4][2] ) 
{
  // Reorder clockwise
  int c [2] = { ( p [0][0] + p [1][0] + p [2][0] + p [3][0] ) / 4, ( p [0][1] + p [1][1] + p [2][1] + p [3][1] ) / 4 };
  double angle [4];
  for ( int i = 0; i < 4; ++i ) {
    angle [i] = atan2 ( p [i][1] - c [1],  p [i][0] - c [0] );
  }

  int perm [4] = { 0, 1, 2, 3 };
  for ( int i = 0; i < 4; ++i )
    for ( int j = i+1; j < 4; ++j )
      if ( angle [perm [i]] > angle [perm [j]] ) {
	int t = perm [i]; perm [i] = perm [j]; perm [j] = t;
      }
      
  int pp [4][2];
  for ( int i = 0; i < 4; ++i )
    for ( int j = 0; j < 2; ++j )
      pp [perm [i]] [j] = p [i][j];

  for ( int i = 0; i < 700; ++i )
    for ( int j = 0; j < 850; ++j ) {
      int x [2] = { i, j };
      if ( inside ( pp, x ) ) {
	img [i][j][0] = 255;
      }
    }
}

// de Castejau Algorithmus: Zeichne die kubische Bezierkurve
// spezifiziert durch die vier Kontrollpunkte p in img ein,
// durch Auswertung der Kurve an n Punkten mit Hilfe des Algorithmus.
void casteljau ( int img [700][850][3], int p [4][2], int n )
{
  for ( int i = 0; i <= n; ++i ) {

    // caste 4 Kontrollpunkte in double Werte,
    // die im de-Casteljau-Algorithmus sukzessive ueberschrieben werden
    double pp [4][2] = { {p[0][0], p[0][1]},  {p[1][0], p[1][1]},  {p[2][0], p[2][1]},  {p[3][0], p[3][1]} };
    double lambda = static_cast<double> ( i ) / n;
    
    // de-Casteljau Algorithmus fuer kubische Bezierkurven:
    // berechne die Auswertung B(\lambda_i)
    for ( int j = 0; j < 3; ++j )
      for ( int k = 0; k < 3-j; ++k ) {
	pp [k][0] = ( 1. - lambda ) * pp [k][0] + lambda * pp [k+1][0];
	pp [k][1] = ( 1. - lambda ) * pp [k][1] + lambda * pp [k+1][1];
      }

    // schreibe B(\lambda_i) in x
    int x [2] = { pp [0][0], pp [0][1] };

    // faerbe x rot in img
    img [x[0]][x[1]][0] = 255;
    img [x[0]][x[1]][1] = 0;
    img [x[0]][x[1]][2] = 0;   
  }
}

// Hauptprogramm
int main () 
{
  // lese Bild ein; speichere x- und y-Groessen
  std::ifstream in ( "loreley.ppm" );
  std::string magic; getline ( in, magic );
  std::string comment; getline ( in, comment );
  int sizex, sizey; in >> sizex >> sizey;
  int max; in >> max;
  
  static int img [700][850][3];
  for ( int i = 0; i < 850; ++i )
    for ( int j = 0; j < 700; ++j )
      for ( int k = 0; k < 3; ++k )
	in >> img [j][849-i][k];

      
  // Spezifiziere stueckweise kubische Bezierkurven durch jeweils vier Kontrollpunkte.
  // Plotte sowohl die vier Kontrollpunkte und deren konvexe Huelle (via plotbox) sowie die Bezierkurve (via casteljau).
  // Verwende zur Bestimmung der Kontrollpunkte das 28x34 - Raster, wobei ein Rasterquadrat 25x25-Pixel umfasst.
      
  // Erste kubische Bezierkurve bestimmt durch die
  // vier Kontrollpunkte in Rasterkoordinaten (jedes Rasterquadrat hat 25x25 Pixel)
  {
    int p [4][2] = { {12*25, 2*25}, {12*25,4*25}, {15*25,7*25}, {17*25,9*25} };
    plotbox ( img, p );
    casteljau ( img, p, 1000 );
  }
  
  // Zweite kubische Bezierkurve
  {
    int p [4][2] = { {17*25,9*25}, {19*25,11*25}, {20*25,12*25}, {20*25,13*25} };
    plotbox ( img, p );
    casteljau ( img, p, 1000 );
  }
  
  // usw.
  {
    int p [4][2] = { {20*25,13*25}, {20*25,14*25}, {19.5*25,15*25}, {19*25,16*25} };
    plotbox ( img, p );
    casteljau ( img, p, 1000 );
  }
  {
    int p [4][2] = { {19*25,16*25}, {18.5*25,17*25}, {18.25*25,18*25}, {17.75*25,18*25} };
    plotbox ( img, p );
    casteljau ( img, p, 1000 );
  }
  {
    int p [4][2] = { {17.75*25,18*25}, {17.25*25,18*25}, {15*25,18.5*25}, {14.5*25,19*25} };
    plotbox ( img, p );
    casteljau ( img, p, 1000 );
  }
  {
    int p [4][2] = { {14.5*25,19*25}, {14*25,19.5*25}, {13*25,22*25}, {13*25,23*25} };
    plotbox ( img, p );
    casteljau ( img, p, 1000 );
  }
  {
    int p [4][2] = { {13*25,23*25}, {13*25,24*25}, {13*25,26.5*25}, {13*25,27*25} };
    plotbox ( img, p );
    casteljau ( img, p, 1000 );
  }
  {
    int p [4][2] = { {13*25,27*25}, {13*25,27.5*25}, {8*25,31*25}, {7*25,31*25} };
    plotbox ( img, p );
    casteljau ( img, p, 1000 );
  }

  // schreibe img in Datei
  std::ofstream out ( "loreleyKurs.ppm" );
  out << magic << std::endl;
  out << sizex << " " << sizey << std::endl;
  out << max << std::endl;
  for ( int i = 0; i < 850; ++i )
    for ( int j = 0; j < 700; ++j )
      for ( int k = 0; k < 3; ++k )
	out << img [j][849-i][k] << std::endl;

  
  return 0;
}
